@@ -36,7 +36,7 @@ import struct
 
 from Cryptodome import Random
 from Cryptodome.IO import PKCS8, PEM
-from Cryptodome.Util.py3compat import tobytes, bord, bchr, b, tostr
+from Cryptodome.Util.py3compat import tobytes, bord, tostr
 from Cryptodome.Util.asn1 import DerSequence
 
 from Cryptodome.Math.Numbers import Integer
@@ -88,7 +88,7 @@ class RsaKey(object):
             The second factor of the modulus. Only required for private keys.
           u : integer
             The CRT coefficient (inverse of p modulo q). Only required for
-            privta keys.
+            private keys.
         """
 
         input_set = set(kwargs.keys())
@@ -96,7 +96,7 @@ class RsaKey(object):
         private_set = public_set | set(('p', 'q', 'd', 'u'))
         if input_set not in (private_set, public_set):
             raise ValueError("Some RSA components are missing")
-        for component, value in list(kwargs.items()):
+        for component, value in kwargs.items():
             setattr(self, "_" + component, value)
 
     @property
@@ -197,10 +197,7 @@ class RsaKey(object):
             return False
         if not self.has_private():
             return True
-        return (self.d == other.d and
-                self.q == other.q and
-                self.p == other.p and
-                self.u == other.u)
+        return (self.d == other.d)
 
     def __ne__(self, other):
         return not (self == other)
@@ -225,7 +222,7 @@ class RsaKey(object):
             key_type = "Public"
         return "%s RSA key at 0x%X" % (key_type, id(self))
 
-    def exportKey(self, format='PEM', passphrase=None, pkcs=1,
+    def export_key(self, format='PEM', passphrase=None, pkcs=1,
                    protection=None, randfunc=None):
         """Export this RSA key.
 
@@ -311,12 +308,12 @@ class RsaKey(object):
         if format == 'OpenSSH':
             e_bytes, n_bytes = [x.to_bytes() for x in (self._e, self._n)]
             if bord(e_bytes[0]) & 0x80:
-                e_bytes = bchr(0) + e_bytes
+                e_bytes = b'\x00' + e_bytes
             if bord(n_bytes[0]) & 0x80:
-                n_bytes = bchr(0) + n_bytes
-            keyparts = [b('ssh-rsa'), e_bytes, n_bytes]
-            keystring = b('').join([struct.pack(">I", len(kp)) + kp for kp in keyparts])
-            return b('ssh-rsa ') + binascii.b2a_base64(keystring)[:-1]
+                n_bytes = b'\x00' + n_bytes
+            keyparts = [b'ssh-rsa', e_bytes, n_bytes]
+            keystring = b''.join([struct.pack(">I", len(kp)) + kp for kp in keyparts])
+            return b'ssh-rsa ' + binascii.b2a_base64(keystring)[:-1]
 
         # DER format is always used, even in case of PEM, which simply
         # encodes it into BASE64.
@@ -361,6 +358,9 @@ class RsaKey(object):
 
         raise ValueError("Unknown key format '%s'. Cannot export the RSA key." % format)
 
+    # Backward compatibility
+    exportKey = export_key
+
     # Methods defined in PyCryptodome that we don't support anymore
     def sign(self, M, K):
         raise NotImplementedError("Use module Cryptodome.Signature.pkcs1_15 instead")
@@ -380,7 +380,7 @@ class RsaKey(object):
     def unblind(self, M, B):
         raise NotImplementedError
 
-    def size():
+    def size(self):
         raise NotImplementedError
 
 
@@ -567,37 +567,43 @@ def construct(rsa_components, consistency_check=True):
         # Build key object
         key = RsaKey(n=n, e=e, d=d, p=p, q=q, u=u)
 
-    # Very consistency of the key
-    fmt_error = False
+    # Verify consistency of the key
     if consistency_check:
+
         # Modulus and public exponent must be coprime
-        fmt_error = e <= 1 or e >= n
-        fmt_error |= Integer(n).gcd(e) != 1
+        if e <= 1 or e >= n:
+            raise ValueError("Invalid RSA public exponent")
+        if Integer(n).gcd(e) != 1:
+            raise ValueError("RSA public exponent is not coprime to modulus")
 
         # For RSA, modulus must be odd
-        fmt_error |= not n & 1
+        if not n & 1:
+            raise ValueError("RSA modulus is not odd")
 
-        if not fmt_error and key.has_private():
+        if key.has_private():
             # Modulus and private exponent must be coprime
-            fmt_error = d <= 1 or d >= n
-            fmt_error |= Integer(n).gcd(d) != 1
+            if d <= 1 or d >= n:
+                raise ValueError("Invalid RSA private exponent")
+            if Integer(n).gcd(d) != 1:
+                raise ValueError("RSA private exponent is not coprime to modulus")
             # Modulus must be product of 2 primes
-            fmt_error |= (p * q != n)
-            fmt_error |= test_probable_prime(p) == COMPOSITE
-            fmt_error |= test_probable_prime(q) == COMPOSITE
+            if p * q != n:
+                raise ValueError("RSA factors do not match modulus")
+            if test_probable_prime(p) == COMPOSITE:
+                raise ValueError("RSA factor p is composite")
+            if test_probable_prime(q) == COMPOSITE:
+                raise ValueError("RSA factor q is composite")
             # See Carmichael theorem
             phi = (p - 1) * (q - 1)
             lcm = phi // (p - 1).gcd(q - 1)
-            fmt_error |= (e * d % int(lcm)) != 1
+            if (e * d % int(lcm)) != 1:
+                raise ValueError("Invalid RSA condition")
             if hasattr(key, 'u'):
                 # CRT coefficient
-                fmt_error |= u <= 1 or u >= q
-                fmt_error |= (p * u % q) != 1
-            else:
-                fmt_error = True
-
-    if fmt_error:
-        raise ValueError("Invalid RSA key components")
+                if u <= 1 or u >= q:
+                    raise ValueError("Invalid RSA component u")
+                if (p * u % q) != 1:
+                    raise ValueError("Invalid RSA component u with p")
 
     return key
 
@@ -670,9 +676,32 @@ def _import_keyDER(extern_key, passphrase):
     raise ValueError("RSA key format is not supported")
 
 
+def _import_openssh_private_rsa(data, password):
+
+    from ._openssh import (import_openssh_private_generic,
+                           read_bytes, read_string, check_padding)
+
+    ssh_name, decrypted = import_openssh_private_generic(data, password)
+
+    if ssh_name != "ssh-rsa":
+        raise ValueError("This SSH key is not RSA")
+
+    n, decrypted = read_bytes(decrypted)
+    e, decrypted = read_bytes(decrypted)
+    d, decrypted = read_bytes(decrypted)
+    iqmp, decrypted = read_bytes(decrypted)
+    p, decrypted = read_bytes(decrypted)
+    q, decrypted = read_bytes(decrypted)
+
+    _, padded = read_string(decrypted)  # Comment
+    check_padding(padded)
+
+    build = [Integer.from_bytes(x) for x in (n, e, d, q, p, iqmp)]
+    return construct(build)
+
+
 def import_key(extern_key, passphrase=None):
-    """Import an RSA key (public or private half), encoded in standard
-    form.
+    """Import an RSA key (public or private).
 
     Args:
       extern_key (string or byte string):
@@ -684,23 +713,19 @@ def import_key(extern_key, passphrase=None):
         - X.509 ``subjectPublicKeyInfo`` DER SEQUENCE (binary or PEM
           encoding)
         - `PKCS#1`_ ``RSAPublicKey`` DER SEQUENCE (binary or PEM encoding)
-        - OpenSSH (textual public key only)
+        - An OpenSSH line (e.g. the content of ``~/.ssh/id_ecdsa``, ASCII)
 
         The following formats are supported for an RSA **private key**:
 
         - PKCS#1 ``RSAPrivateKey`` DER SEQUENCE (binary or PEM encoding)
         - `PKCS#8`_ ``PrivateKeyInfo`` or ``EncryptedPrivateKeyInfo``
           DER SEQUENCE (binary or PEM encoding)
-        - OpenSSH (textual public key only)
+        - OpenSSH (text format, introduced in `OpenSSH 6.5`_)
 
         For details about the PEM encoding, see `RFC1421`_/`RFC1423`_.
 
-        The private key may be encrypted by means of a certain pass phrase
-        either at the PEM level or at the PKCS#8 level.
-
-      passphrase (string):
-        In case of an encrypted private key, this is the pass phrase from
-        which the decryption key is derived.
+      passphrase (string or byte string):
+        For private keys only, the pass phrase that encrypts the key.
 
     Returns: An RSA key object (:class:`RsaKey`).
 
@@ -713,37 +738,45 @@ def import_key(extern_key, passphrase=None):
     .. _RFC1423: http://www.ietf.org/rfc/rfc1423.txt
     .. _`PKCS#1`: http://www.ietf.org/rfc/rfc3447.txt
     .. _`PKCS#8`: http://www.ietf.org/rfc/rfc5208.txt
+    .. _`OpenSSH 6.5`: https://flak.tedunangst.com/post/new-openssh-key-format-and-bcrypt-pbkdf
     """
 
     extern_key = tobytes(extern_key)
     if passphrase is not None:
         passphrase = tobytes(passphrase)
 
-    if extern_key.startswith(b('-----')):
+    if extern_key.startswith(b'-----BEGIN OPENSSH PRIVATE KEY'):
+        text_encoded = tostr(extern_key)
+        openssh_encoded, marker, enc_flag = PEM.decode(text_encoded, passphrase)
+        result = _import_openssh_private_rsa(openssh_encoded, passphrase)
+        return result
+
+    if extern_key.startswith(b'-----'):
         # This is probably a PEM encoded key.
         (der, marker, enc_flag) = PEM.decode(tostr(extern_key), passphrase)
         if enc_flag:
             passphrase = None
         return _import_keyDER(der, passphrase)
 
-    if extern_key.startswith(b('ssh-rsa ')):
-            # This is probably an OpenSSH key
-            keystring = binascii.a2b_base64(extern_key.split(b(' '))[1])
-            keyparts = []
-            while len(keystring) > 4:
-                l = struct.unpack(">I", keystring[:4])[0]
-                keyparts.append(keystring[4:4 + l])
-                keystring = keystring[4 + l:]
-            e = Integer.from_bytes(keyparts[1])
-            n = Integer.from_bytes(keyparts[2])
-            return construct([n, e])
+    if extern_key.startswith(b'ssh-rsa '):
+        # This is probably an OpenSSH key
+        keystring = binascii.a2b_base64(extern_key.split(b' ')[1])
+        keyparts = []
+        while len(keystring) > 4:
+            length = struct.unpack(">I", keystring[:4])[0]
+            keyparts.append(keystring[4:4 + length])
+            keystring = keystring[4 + length:]
+        e = Integer.from_bytes(keyparts[1])
+        n = Integer.from_bytes(keyparts[2])
+        return construct([n, e])
 
-    if bord(extern_key[0]) == 0x30:
-            # This is probably a DER encoded key
-            return _import_keyDER(extern_key, passphrase)
+    if len(extern_key) > 0 and bord(extern_key[0]) == 0x30:
+        # This is probably a DER encoded key
+        return _import_keyDER(extern_key, passphrase)
 
     raise ValueError("RSA key format is not supported")
 
+
 # Backward compatibility
 importKey = import_key
 
